" Vim syntax file
" Language:     Assembler, KickAssembler
" Maintainer:   Roman 'gryf' Dobosz <gryf_esm@o2.pl>
" Last Change:  2012-06-03
" Version:      0.1
"
" To install this file place it in ~/.vim/syntax (*nix/Mac) or in
" $VIMRUNTIME/syntax on Windows and issue command:
"
" :set filetype=kickass
"
" You can also add to your vimrc file autocommand:
"
" autocmd BufRead *.asm set filetype=kickass
"
" From now on, all files with extension 'asm' will have brand new kickass
" syntax.
"
" Enjoy.

syn clear
syn case ignore

syn region asmString  start=+"+ end=+"+
syn region asmSString start=+'+ end=+'+ contains=@Spell

syn keyword asm6502Mnemonics    adc and asl bit brk clc cld cli clv cmp cpx cpy
syn keyword asm6502Mnemonics    dec dex dey eor inc inx iny lda ldx ldy lsr nop
syn keyword asm6502Mnemonics    ora pha php pla plp rol ror rti rts sbc sec sed
syn keyword asm6502Mnemonics    sei sta stx sty tax tay tsx txa txs tya jmp jsr

syn match asm6502Index  ",\s*[xy]" contains=asm6502Mnemonics

syn keyword asm6502Branch   bcc bcs beq bmi bne bpl bvc bvs

syn keyword asm6510Illegal  slo rla sre rra sax lax dcp isc anc asr arr sbx
syn keyword asm6510Illegal  dop top jam

syn match asmMacroCall      ":[a-z_][a-z0-9_]*"
syn match asmReferenceCall  "\~[a-z_+-.][a-z0-9_+-]*"

syn keyword asmTodo         TODO FIXME XXX TBD NOTE WARNING BUG
syn match   asmLineComment  "\/\/.*" contains=@Spell,asmTodo
syn region  asmComment      start="/\*" end="\*/" contains=@Spell,asmTodo

syn region asmString    start=+"+ skip=+\\"+ end=+"+ contains=@Spell
syn region asmSString   start=+'+ skip=+\\'+ end=+'+ contains=@Spell

syn match asmLabel  "\!\?\<[a-z_+-.][a-z0-9_+-]*\>[:+-]\?"

syn match decNumber     "\<\d\+\>"
syn match hexNumber     "\$\x\+\>"
syn match binNumber     "%[01]\+\>"
syn match asmImmediate  "#\$\x\+\>"
syn match asmImmediate  "#\d\+\>"
syn match asmImmediate  "<\$\x\+\>"
syn match asmImmediate  "<\d\+\>"
syn match asmImmediate  ">\$\x\+\>"
syn match asmImmediate  ">\d\+\>"
syn match asmImmediate  "#<\$\x\+\>"
syn match asmImmediate  "#<\d\+\>"
syn match asmImmediate  "#>\$\x\+\>"
syn match asmImmediate  "#>\d\+\>"

syn match asmKickKeyword   "\.\<byte\>"
syn match asmKickKeyword   "\.\<define\>"
syn match asmKickKeyword   "\.\<eval\>"
syn match asmKickKeyword   "\.\<filenamespace\>"
syn match asmKickKeyword   "\.\<fill\>"
syn match asmKickKeyword   "\.\<for\>"
syn match asmKickKeyword   "\.\<function\>" nextgroup=asmDefName skipwhite
syn match asmKickKeyword   "\.\<if\>"
syn match asmKickKeyword   "\.\<import\>"
syn match asmKickKeyword   "\.\<label\>"
syn match asmKickKeyword   "\.\<macro\>" nextgroup=asmDefName skipwhite
syn match asmKickKeyword   "\.\<namespace\>"
syn match asmKickKeyword   "\.\<pc\>"
syn match asmKickKeyword   "\.\<print\>"
syn match asmKickKeyword   "\.\<pseudocommand\>"
syn match asmKickKeyword   "\.\<return\>"
syn match asmKickKeyword   "\.\<text\>"
syn match asmKickKeyword   "\.\<var\>"
syn match asmKickKeyword   "\<else\>"

syn match   asmDefName  "[a-zA-Z_][a-zA-Z0-9_]*" display contained

if !exists("did_kickasm_syntax_inits")
  let did_kickasm_syntax_inits = 1

    hi def link asmKickKeyword      Special
    hi def link asm6510Illegal      Debug
    hi def link asm6502Mnemonics    Type
    hi def link asm6502Index        None
    hi def link asm6502Branch       Conditional
    hi def link asmString           String
    hi def link asmSString          String
    hi def link asmComment          Comment
    hi def link asmLineComment      Comment
    hi def link asmMacroCall        Function
    hi def link asmReferenceCall    Debug
    hi def link asmLabel            Label
    hi def link asmTodo             Todo

    hi def link asmDefName          Function

    hi def link asmImmediate        None
    hi def link hexNumber           None
    hi def link binNumber           None
    hi def link decNumber           None

endif

let b:current_syntax = "kickasm"
